require 'assent/validations/options'
require 'date'

module Assent
  class Validate
    include Options

    def initialize(field, value, rules, errors)
      @field = field
      @value = value
      @rules = rules
      @errors = errors
    end
    # Validate method that checks all the validations for a single field
    def validate
      required
      accepted

      return if blank?

      date
      ip

      @errors
    end

    private

    # Validates the presence of a field
    #
    # Example:
    #
    # class LoginValidator < Assent::Validator
    #   field :email, presence: true
    # end
    #
    # credentials = Hash.new
    # credentials[:email] = nil
    # LoginValidator.new.errors will contain errors because of a nil value
    #
    #
    # credentials[:email] = "test@example.com"
    # This will pass
    def required
      validation(:presence) { !blank? }
    end

    # Validates if a field has the value true, 1 or yes
    #
    # Example:
    # class TermsValidator < Assent::Validator
    #   field :terms, accepted: true
    # end
    def accepted
      validation(:accepted) { @value == true || @value == 1}
    end

    # Validates if a field is a valid date
    #
    # Example:
    # class RegistrationValidator < Assent::validator
    # field :date, date: true
    # end
    def date
      validation(:date) do |validator|
        date = Date.parse(@value) rescue nil

        !date.nil?
      end
    end

    # Validates if a field is a valid ip-address
    #
    # Example:
    # class IpValidator < Assent::Validator
    #   field :ip, ip: true
    # end
    def ip
      validation(:ip) do |validator|
        result = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/.match(@value.to_s)

        !result.nil?
      end
    end

    # Checks if the given value is nil or empty
    def blank?
      @value.nil? || @value.empty?
    end

    def validation(rule)
      validator = @rules[rule]

      if(validator) && (!yield validator)
        @errors.add(@field, rule)
      end

      yield validator
    end
  end
end
